unit MFCustomPalette;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls,IniFiles;

type
  TColumns = (columns2,columns4,columns8);

type TCellClickEvent = procedure (Index : Integer;Button : TMouseButton;
                                  Color : TColor) of object;

type
 TCell = class(TCustomPanel)
 private
  CellIndex : Integer;
 protected
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
                        X,Y : Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
                        X,Y : Integer); override;
 public
  Constructor Create(AOwner : TComponent); override;
end;

type
  TCustomPalette = class(TCustomPanel)
  private
    FOnCellClick : TCellCLickEvent;
    FColumns : TColumns;
    CellWidth,CellHeight : Integer;
    AdjustWidth,AdjustHeight : Integer;
    Cells : array[0..15] of TCell;
    Colors : array[0..15] of TColor;
    IniFile : TIniFile;
    procedure SetColumns(Num : TColumns);
 
  protected
    procedure PositionCells;
    procedure Resize; override;
  public
    FCellClicked : Integer;
    constructor Create(AOwner : TComponent); override;
    procedure SetStandardColors;
    procedure SetCellColor(index : integer;Color : TColor);
    function  GetCellColor(index : integer) : TColor;
    procedure SaveToFile(name : String);
    procedure LoadFromFile(name : string);
    procedure DoCellClick(Button : TMouseButton;Index : Integer;Color : TColor);
  published
    Property Enabled;
    Property Visible;
    property OnResize;
    property Columns : TColumns read FColumns write SetColumns default columns4;
    property OnCellClick : TCellClickEvent read FOnCellClick write FOnCellClick;
  end;

procedure Register;

implementation


{$R MFCustomPalette}
const TheCursor = 5000;

procedure Register;
begin
  RegisterComponents('MicroFrame', [TCustomPalette]);
end;

constructor TCell.Create(AOwner : TComponent);
begin
 inherited Create(AOwner);
 Width := 10;
 Height := 10;
 BevelInner := bvLowered;
 BorderStyle := bsSingle;
 Cursor := TheCursor;

end;

procedure TCell.MouseDown(Button: TMouseButton; Shift: TShiftState;
                        X,Y : Integer);
begin
 (Parent as TCustomPalette).FCellClicked := CellIndex;
end;

procedure TCell.MouseUp(Button: TMouseButton; Shift: TShiftState;
                        X,Y : Integer);
begin
  if CellIndex = (Parent as TCustomPalette).FCellClicked then begin
   (Parent as TCustomPalette).DoCellClick(Button,CellIndex,Color);
  end;
end;


constructor TCustomPalette.Create(AOwner : TComponent);
var
 i : integer;
begin
 inherited Create(AOwner);
 Width := 100;
 Height := 100;
 FColumns := columns4;
 Screen.Cursors[TheCursor] := LoadCursor(hInstance,pChar('PICKER'));
 for i := 0 to 15 do begin
  Cells[i] := TCell.Create(self);
  Cells[i].Parent := self;
  Cells[i].CellIndex := i;
 end;
 SetStandardColors;
 Resize;
end;

procedure TCustomPalette.Resize;
begin
 PositionCells;
 if AdjustHeight <> 0 then Height := AdjustHeight;
 if AdjustWidth <> 0 then  Width := AdjustWidth;
 AdjustWidth := 0;
 AdjustHeight := 0;
 inherited Resize;
end;

procedure TCustomPalette.SetStandardColors;
var
 i : integer;
begin
 Cells[0].Color := clBlack;
 Cells[1].Color := clMaroon;
 Cells[2].Color := clGreen;
 Cells[3].Color := clOlive;
 Cells[4].Color := clNavy;
 Cells[5].Color := clPurple;
 Cells[6].Color := clTeal;
 Cells[7].Color := clGray;
 Cells[8].Color := clSilver;
 Cells[9].Color := clRed;
 Cells[10].Color := clLime;
 Cells[11].Color := clYellow;
 Cells[12].Color := clBlue;
 Cells[13].Color := clFuchsia;
 Cells[14].Color := clAqua;
 Cells[15].Color := clWhite;
 for i := 0 to 15 do begin
  Colors[i] := Cells[i].Color;
 end;
end;

procedure TCustomPalette.PositionCells;
var
 i : integer;
begin
  AdjustWidth := 0;
  AdjustHeight := 0;
  if FColumns = columns2 then begin
   CellWidth := Width div 2;
   CellHeight := Height div 8;
   for i := 0 to 15 do begin
    Cells[i].Width := CellWidth;
    Cells[i].Height := CellHeight;
   end;
   Cells[0].Left := 0;
   Cells[0].Top := 0;
   Cells[1].Left := CellWidth;
   Cells[1].Top := 0;
   Cells[2].Left := 0;
   Cells[2].Top := CellHeight;
   Cells[3].Left := CellWidth;
   Cells[3].Top := CellHeight;
   Cells[4].Left := 0;
   Cells[4].Top := CellHeight * 2;
   Cells[5].Left := CellWidth;
   Cells[5].Top := CellHeight * 2;
   Cells[6].Left := 0;
   Cells[6].Top := CellHeight * 3;
   Cells[7].Left := CellWidth;
   Cells[7].Top := CellHeight * 3;
   Cells[8].Left := 0;
   Cells[8].Top := CellHeight * 4;
   Cells[9].Left := CellWidth;
   Cells[9].Top := CellHeight * 4;
   Cells[10].Left := 0;
   Cells[10].Top := CellHeight * 5;
   Cells[11].Left := CellWidth;
   Cells[11].Top := CellHeight * 5;
   Cells[12].Left := 0;
   Cells[12].Top := CellHeight * 6;
   Cells[13].Left := CellWidth;
   Cells[13].Top := CellHeight * 6;
   Cells[14].Left := 0;
   Cells[14].Top := CellHeight * 7;
   Cells[15].Left := CellWidth;
   Cells[15].Top := CellHeight * 7;
   if Height > Cells[14].Top + CellHeight then begin
    AdjustHeight := Cells[14].Top + CellHeight;
   end;
  end;
  if FColumns = columns4 then begin
   CellWidth := Width div 4;
   CellHeight := Height div 4;
   for i := 0 to 15 do begin
    Cells[i].Width := CellWidth;
    Cells[i].Height := CellHeight;
   end;
   Cells[0].Left := 0;
   Cells[0].Top := 0;
   Cells[1].Left := CellWidth;
   Cells[1].Top := 0;
   Cells[2].Left := CellWidth * 2;
   Cells[2].Top := 0;
   Cells[3].Left := CellWidth * 3;
   Cells[3].Top := 0;
   Cells[4].Left := 0;
   Cells[4].Top := CellHeight;
   Cells[5].Left := CellWidth;
   Cells[5].Top := CellHeight;
   Cells[6].Left := CellWidth * 2;
   Cells[6].Top := CellHeight;
   Cells[7].Left := CellWidth * 3;
   Cells[7].Top := CellHeight;
   Cells[8].Left := 0;
   Cells[8].Top := CellHeight * 2;
   Cells[9].Left := CellWidth;
   Cells[9].Top := CellHeight * 2;
   Cells[10].Left := CellWidth * 2;
   Cells[10].Top := CellHeight * 2;
   Cells[11].Left := CellWidth * 3;
   Cells[11].Top := CellHeight * 2;
   Cells[12].Left := 0;
   Cells[12].Top := CellHeight * 3;
   Cells[13].Left := CellWidth;
   Cells[13].Top := CellHeight * 3;
   Cells[14].Left := CellWidth * 2;
   Cells[14].Top := CellHeight * 3;
   Cells[15].Left := CellWidth * 3;
   Cells[15].Top := CellHeight * 3;
  end;
  if FColumns = columns8 then begin
   CellWidth := Width div 8;
   CellHeight := Height div 2;
   for i := 0 to 15 do begin
    Cells[i].Width := CellWidth;
    Cells[i].Height := CellHeight;
   end;
   Cells[0].Left := 0;
   Cells[0].Top := 0;
   Cells[1].Left := CellWidth;
   Cells[1].Top := 0;
   Cells[2].Left := CellWidth * 2;
   Cells[2].Top := 0;
   Cells[3].Left := CellWidth * 3;
   Cells[3].Top := 0;
   Cells[4].Left := CellWidth * 4;
   Cells[4].Top := 0;
   Cells[5].Left := CellWidth * 5;
   Cells[5].Top := 0;
   Cells[6].Left := CellWidth * 6;
   Cells[6].Top := 0;
   Cells[7].Left := CellWidth * 7;
   Cells[7].Top := 0;
   Cells[8].Left := 0;
   Cells[8].Top := CellHeight;
   Cells[9].Left := CellWidth;
   Cells[9].Top := CellHeight;
   Cells[10].Left := CellWidth * 2;
   Cells[10].Top := CellHeight;
   Cells[11].Left := CellWidth * 3;
   Cells[11].Top := CellHeight;
   Cells[12].Left := CellWidth * 4;
   Cells[12].Top := CellHeight;
   Cells[13].Left := CellWidth * 5;
   Cells[13].Top := CellHeight;
   Cells[14].Left := CellWidth * 6;
   Cells[14].Top := CellHeight;
   Cells[15].Left := CellWidth * 7;
   Cells[15].Top := CellHeight;
   if Width > Cells[7].Left + CellWidth then begin
    AdjustWidth := Cells[7].Left + CellWidth;
   end;
  end;
end;



procedure TCustomPalette.SetColumns(Num : TColumns);
begin
 FColumns := Num;
 Resize;
end;

procedure TCustomPalette.SetCellColor(index : integer;Color : TColor);
begin
 Colors[index] := Color;
 Cells[index].Color := Color;
end;

function TCustomPalette.GetCellColor(index : integer) : TColor;
begin
 result := Cells[index].Color;
end;

procedure TCustomPalette.DoCellClick(Button : TMouseButton;
                                     Index : Integer;Color : TColor);
begin
  if Assigned(FOnCellClick) then
   FOnCellClick(Index,Button,Color);
end;

procedure TCustomPalette.SaveToFile(name : String);
begin
 IniFile := TIniFile.Create(name);
 IniFile.WriteInteger('Colors','Color00',Cells[0].Color);
 IniFile.WriteInteger('Colors','Color01',Cells[1].Color);
 IniFile.WriteInteger('Colors','Color02',Cells[2].Color);
 IniFile.WriteInteger('Colors','Color03',Cells[3].Color);
 IniFile.WriteInteger('Colors','Color04',Cells[4].Color);
 IniFile.WriteInteger('Colors','Color05',Cells[5].Color);
 IniFile.WriteInteger('Colors','Color06',Cells[6].Color);
 IniFile.WriteInteger('Colors','Color07',Cells[7].Color);
 IniFile.WriteInteger('Colors','Color08',Cells[8].Color);
 IniFile.WriteInteger('Colors','Color09',Cells[9].Color);
 IniFile.WriteInteger('Colors','Color10',Cells[10].Color);
 IniFile.WriteInteger('Colors','Color11',Cells[11].Color);
 IniFile.WriteInteger('Colors','Color12',Cells[12].Color);
 IniFile.WriteInteger('Colors','Color13',Cells[13].Color);
 IniFile.WriteInteger('Colors','Color14',Cells[14].Color);
 IniFile.WriteInteger('Colors','Color15',Cells[15].Color);
 IniFile.Free;
end;

procedure TCustomPalette.LoadFromFile(name : String);
begin
  if not FileExists(name) then exit;
  IniFile := TIniFile.Create(name);
  Cells[0].Color := IniFile.ReadInteger('Colors','Color00',0);
  Cells[1].Color := IniFile.ReadInteger('Colors','Color01',0);
  Cells[2].Color := IniFile.ReadInteger('Colors','Color02',0);
  Cells[3].Color := IniFile.ReadInteger('Colors','Color03',0);
  Cells[4].Color := IniFile.ReadInteger('Colors','Color04',0);
  Cells[5].Color := IniFile.ReadInteger('Colors','Color05',0);
  Cells[6].Color := IniFile.ReadInteger('Colors','Color06',0);
  Cells[7].Color := IniFile.ReadInteger('Colors','Color07',0);
  Cells[8].Color := IniFile.ReadInteger('Colors','Color08',0);
  Cells[9].Color := IniFile.ReadInteger('Colors','Color09',0);
  Cells[10].Color := IniFile.ReadInteger('Colors','Color10',0);
  Cells[11].Color := IniFile.ReadInteger('Colors','Color11',0);
  Cells[12].Color := IniFile.ReadInteger('Colors','Color12',0);
  Cells[13].Color := IniFile.ReadInteger('Colors','Color13',0);
  Cells[14].Color := IniFile.ReadInteger('Colors','Color14',0);
  Cells[15].Color := IniFile.ReadInteger('Colors','Color15',0);
  IniFile.Free;
end;


end.
